<?php

class RCCatalog {

	public function __construct () {
		
		$this->visibleCount = 0;
		
		$this->buildProductsList();
		$this->buildCategoriesAndTagsLists();
		$this->buildSortOptions();
	
	}
	
	private function buildProductsList () {
		
		$this->products = array();
		
		foreach (RCUtilities::getJSONData('products.json') as $data) {
			
			$product = new RCProduct($data);
			
			$this->products[] = $product;
			
			if (!isset($product->hidden) || !$product->hidden) {
				
				$this->visibleCount++;
				
			}
		
		}
		
		if (RCUtilities::$settings->product->maxRelatedProducts > 0) {
			
			$this->buildRelatedProducts();
			
		}
		
		$this->allProducts = $this->products;
		
	}
	
	public function resetProductsList () {
	
		$this->products = $this->allProducts;
		
	}
	
	private function buildCategoriesAndTagsLists () {
		
		$this->categories = array();
		$this->tags = array();
		
		foreach ($this->products as $product) {
			
			if (!isset($product->hidden) || !$product->hidden) {
			
				foreach ($product->categories as $product_category) {
				
					$found = FALSE;
					
					foreach ($this->categories as $category) {
						
						if ($category->ID == $product_category->ID) {
							
							$found = TRUE;
							break;
							
						}
						
					}
				
					if (!$found) {
						
						$this->categories[] = (object) array(
						
							'ID'       => $product_category->ID,
							'name'     => $product_category->name,
							'link'     => RCUtilities::getURL(array(
							
								1 => 'category',
								2 => urlencode($product_category->ID),
								4 => 1
								
							)),
							'selected' => FALSE
							
						);
						
					}
					
				}
				
				if (RCUtilities::$settings->catalog->sortToolbarCategories) {
					
					usort($this->categories, function ($a, $b) {
					
						return strnatcmp($a->name, $b->name);
						
					});
					
				}
				
				foreach ($product->tags as $product_tag) {
				
					$found = FALSE;
					
					foreach ($this->tags as $tag) {
						
						if ($tag->ID == $product_tag->ID) {
							
							$found = TRUE;
							break;
							
						}
						
					}
				
					if (!$found) {
				
						$this->tags[] = (object) array(
						
							'ID'       => $product_tag->ID,
							'name'     => $product_tag->name,
							'link'     => RCUtilities::getURL(array(
							
								1 => 'tag',
								2 => urlencode($product_tag->ID),
								4 => 1
								
							)),
							'selected' => FALSE
							
						);
						
					}
					
				}
				
				if (RCUtilities::$settings->catalog->sortToolbarTags) {
					
					usort($this->tags, function ($a, $b) {
					
						return strnatcmp($a->name, $b->name);
						
					});
					
				}
				
			}
		
		}
		
	}
	
	private function buildSortOptions () {
		
		$sort = array('date', 'name');
		$icons = array(
			
			'date'   => 'sort-amount-desc',
			'name'   => 'sort-alpha-asc',
			'price'  => 'sort-numeric-asc',
			'rating' => 'sort-amount-desc'
			
		);
		
		if (RCUtilities::$settings->catalog->showStartingPrices) {
			
			$sort[] = 'price';
			
		}
		
		if (RCUtilities::$settings->database->productRatingEnabled) {
			
			$sort[] = 'rating';
			
		}
		
		$this->sort = array();
		
		foreach ($sort as $value) {
		
			$this->sort[] = (object) array(
			
				'ID'       => $value,
				'link'     => RCUtilities::getURL(array(
				
					3 => $value
					
				)),
				'selected' => FALSE,
				'icon'     => $icons[$value]
				
			);
			
		}
		
	}
	
	private function buildRelatedProducts () {
			
		$products = $this->products;
		shuffle($products);
			
		usort($products, function ($a, $b) {
			
			if ($a->promoted > $b->promoted) {
				
				return -1;
				
			}
			
			if ($b->promoted > $a->promoted) {
				
				return 1;
				
			}
			
			return 0;
			
		});
		
		foreach ($this->products as $product) {
			
			$product->related = array();
			
			foreach ($products as $related) {
				
				if ($product->ID != $related->ID) {
					
					$match = array_uintersect($product->{RCUtilities::$settings->product->showRelatedProducts}, $related->{RCUtilities::$settings->product->showRelatedProducts}, function ($a, $b) {
						
						return $a->ID != $b->ID;
						
					});
					
					if (!empty($match) && (count($product->related) < RCUtilities::$settings->product->maxRelatedProducts)) {
						
						$product->related[] = (object) array(
							
							'image'         => $related->images[0]->thumbnailPath,
							'link'          => $related->link,
							'name'          => $related->name,
							'description'   => $related->description->short,
							'tags'          => $related->tags,
							'promoted'      => $related->promoted,
							'ratingAverage' => $related->ratingAverage,
							'ratingCount'   => $related->ratingCount
						
						);
						
					}
					
				}
				
			}
			
		}
		
	}
	
	public function getProduct ($ID = NULL) {
		
		$this->resetProductsList();
		
		if (is_null($ID)) {
		
			return $this->products[array_rand($this->products)];
			
		}
	
		foreach ($this->products as $product) {
			
			if ($product->ID == $ID) {
				
				return $product;
				
			}
			
		}
		
		return FALSE;
		
	}
	
	public function applyFilter ($type, $value) {
	
		$filter = FALSE;
		$products = array();
	
		$link = RCUtilities::getURL(array(

			1 => 'all',
			2 => '-'
			
		));
		
		switch ($type) {
			
			case 'category':
			
				foreach ($this->categories as $i => $category) {
					
					if ($category->ID == $value) {
						
						$this->categories[$i]->selected = TRUE;
						$this->categories[$i]->link = $link;
						
					}
					
				}
				
				foreach ($this->products as $product) {
					
					if (!isset($product->hidden) || !$product->hidden) {
					
						foreach ($product->categories as $category) {
							
							if ($category->ID == $value) {
								
								$products[] = $product;
								break;
								
							}
							
						}
						
					}
					
				}
			
			break;
			
			case 'tag':
			
				foreach ($this->tags as $i => $tag) {
					
					if ($tag->ID == $value) {
						
						$this->tags[$i]->selected = TRUE;
						$this->tags[$i]->link = $link;
						
					}
					
				}
				
				foreach ($this->products as $product) {
					
					if (!isset($product->hidden) || !$product->hidden) {
						
						foreach ($product->tags as $tag) {
							
							if ($tag->ID == $value) {
								
								$products[] = $product;
								break;
								
							}
							
						}
						
					}
					
				}
			
			break;
			
			case 'search':
			
				$words = explode(' ' , $value);
				$words = array_filter($words);
			
				foreach ($this->products as $product) {
					
					if (!isset($product->hidden) || !$product->hidden) {
						
						$matchAll = TRUE;
						
						foreach ($words as $value) {
							
							$match = stripos($product->name, $value) !== FALSE ||
							         stripos($product->description->short, $value) !== FALSE ||
							         stripos($product->description->long, $value) !== FALSE;
							
							if (!$match) {
								
								foreach ($product->versions as $version) {
									
									if (stripos($version->name, $value) !== FALSE ||
									    stripos($version->SKU, $value) !== FALSE) {
										
										$match = TRUE;
										break;
										
									}
									
								}
								
							}
								
							if (!$match) {
								
								foreach ($product->categories as $category) {
									
									if (stripos($category->ID, $value) !== FALSE) {
										
										$match = TRUE;
										break;
										
									}
									
								}
								
							}
								
							if (!$match) {
								
								foreach ($product->tags as $tag) {
									
									if (stripos($tag->ID, $value) !== FALSE) {
										
										$match = TRUE;
										break;
										
									}
									
								}
								
							}
							
							if (!$match) {
								
								$matchAll = FALSE;
								
							}
							
						}
					
						if ($matchAll) {
							
							$products[] = $product;
							
						}
						
					}
					
				}
			
			break;
			
			default:
			
				return;
				
			break;
			
		}
			
		$this->visibleCount = count($products);
		$this->products = $products;
		
	}
	
	public function sortProducts ($ID) {
	
		foreach ($this->sort as $i => $sort) {
			
			if ($sort->ID == $ID) {
				
				$this->sort[$i]->selected = TRUE;
				break;
				
			}
			
		}
	
		switch ($ID) {
		
			case 'date':
			
				usort($this->products, function ($a, $b) {
					
					if ($a->promoted > $b->promoted) {
						
						return -1;
						
					}
					
					if ($b->promoted > $a->promoted) {
						
						return 1;
						
					}
					
					if ($a->publishDate > $b->publishDate) {
						
						return -1;
						
					}
					
					if ($b->publishDate > $a->publishDate) {
						
						return 1;
						
					}
				
					return 0;
					
				});
				
			break;
		
			case 'name':
			
				usort($this->products, function ($a, $b) {
					
					if ($a->promoted > $b->promoted) {
						
						return -1;
						
					}
					
					if ($b->promoted > $a->promoted) {
						
						return 1;
						
					}
				
					return strnatcmp(strtolower($a->name), strtolower($b->name));
					
				});
				
			break;
			
			case 'price':
			
				usort($this->products, function ($a, $b) {
					
					if ($a->promoted > $b->promoted) {
						
						return -1;
						
					}
					
					if ($b->promoted > $a->promoted) {
						
						return 1;
						
					}
					
					if ($a->basePrice > $b->basePrice) {
						
						return 1;
						
					}
					
					if ($b->basePrice > $a->basePrice) {
						
						return -1;
						
					}
				
					return 0;
					
				});
				
			break;
		
			case 'rating':
			
				usort($this->products, function ($a, $b) {
					
					if ($a->promoted > $b->promoted) {
						
						return -1;
						
					}
					
					if ($b->promoted > $a->promoted) {
						
						return 1;
						
					}
					
					if ($a->ratingAverage > $b->ratingAverage) {
						
						return -1;
						
					}
					
					if ($b->ratingAverage > $a->ratingAverage) {
						
						return 1;
						
					}
				
					return 0;
					
				});
				
			break;
			
		}
		
	}
	
	public function getProductsPage ($page) {
		
		$from = RCUtilities::$settings->catalog->productsPerPage * ($page - 1);
		$to = min(RCUtilities::$settings->catalog->productsPerPage * $page, $this->visibleCount);
		
		$visibleProducts = array();
		
		foreach ($this->products as $product) {
			
			if (!isset($product->hidden) || !$product->hidden) {
				
				$visibleProducts[] = $product;
				
			}
			
		}
		
		$products = array();
		
		for ($i = $from; $i < $to; $i++) {
			
			$products[] = $visibleProducts[$i];
			
		}
		
		return $products;
		
	}
	
	public function validateProducts ($cart) {
	
		$invalid = array();
		
		foreach ($cart as $item) {
		
			$found = FALSE;
			
			foreach ($this->products as $product) {
				
				if ($item->product->validityID == $product->validityID) {
					
					$found = TRUE;
					break;
					
				}
				
			}
			
			if (!$found) {
				
				$invalid[] = $item->ID;
				
			}
			
		}
		
		return $invalid;
		
	}
	
	public function getDownloadLinks ($SKU) {
		
		$download = FALSE;
		
		foreach ($this->products as $product) {
			
			foreach ($product->versions as $version) {
				
				if ($version->SKU == $SKU && $version->delivery->digital && isset($version->delivery->sources) && isset($version->delivery->sources->AWS)) {
					
					$download = (object) array(
						
						'name'    => $product->name,
						'version' => $version->name,
						'SKU'     => $SKU,
						'links'   => array()
						
					);
					
					if (!defined('CURL_SSLVERSION_TLSv1')) {
						
						define('CURL_SSLVERSION_TLSv1', 1);
						
					}
				
					S3::setAuth(RCUtilities::$config->amazon->accessKey, RCUtilities::$config->amazon->secretKey);
					
					foreach ($version->delivery->sources->AWS->buckets as $bucket) {
						
						foreach ($bucket->files as $file) {
							
							$info = S3::getObjectInfo($bucket->name, $file);
				
							$download->links[] = (object) array(
							
								'name' => basename($file),
								'type' => $info['type'],
								'size' => RCUtilities::formatBytes($info['size']),
								'URL'  => S3::getAuthenticatedURL($bucket->name, $file, 60 * 60)
								
							);
							
						}
						
					}
					
				}
				
			}
			
		}
		
		return $download;
		
	}
	
}

?>