<?php

class RCController {

	public function __construct ($autoRedirection = TRUE) {

		if (!defined('RC_DEBUG_MODE')) {

			define('RC_DEBUG_MODE', FALSE);

		}

		if (RC_DEBUG_MODE) {

			$startExecTime = microtime(TRUE);

		}

		// inizializza impostazioni, localizzazione ed altre configurazioni
		RCUtilities::init();

		$this->header = $this->loadTemplate('common/header', array(

			'style' => (RCUtilities::$settings->general->style != 'flat') ? ('.' . RCUtilities::$settings->general->style) : '',
			'path'  => RCUtilities::$settings->general->useCDN ? 'https://cdnjs.cloudflare.com/ajax/libs/' : (RC_PLUGIN_FOLDER . '/resources/')

		));
		$this->page = '';

		$this->databaseEnabled = FALSE;

		if (RCUtilities::$settings->database->enable) {

			$this->database = new RCDatabase();
			$this->databaseEnabled = empty($this->database->error);

			if (!$this->databaseEnabled) {

				$this->showDebug($this->database->error);

			}

		}

		RCUtilities::$settings->database->productRatingEnabled = RCUtilities::$settings->database->productRatingEnabled && $this->databaseEnabled;

		if ($this->databaseEnabled) {

			$this->database->syncCoupons();

		}

		$this->catalog = new RCCatalog();
		$this->cart = new RCCart();

		if ($this->databaseEnabled) {

			$this->database->syncProducts($this->catalog, $this->cart);

		}

		$this->cart->removeItems($this->catalog->validateProducts($this->cart->items));

		if (RCUtilities::$settings->database->productRatingEnabled) {

			$this->database->getRatings($this->catalog);

		}

		$this->setURLs();
		$this->setExtras();

		if (RCUtilities::$settings->dealers->loggedIn) {

			$this->page .= $this->loadTemplate('common/dealers');

		}

		if ($autoRedirection) {

			$this->getPage();

		}

		if (RC_DEBUG_MODE) {

			$this->page .= $this->showDebug('Page loaded in ' . round(microtime(true) - $startExecTime, 3) . ' secs.');

		}

	}

	// carica il template in memoria
	private function loadTemplate ($view, $options = array()) {

		if (is_array($options)) {

			extract($options);

		}

		ob_start();

		include dirname(__FILE__) . '/../templates/' . $view . '.php';

		$result = ob_get_clean();

		return $result;

	}

	// mostra il contenuto come pagina JSON
	private function sendJSON ($data = FALSE, $noExtras = FALSE) {

		if (ob_get_contents()) {

			ob_end_clean();

		}

		header('Content-type: application/json');

		if ($noExtras) {

			echo json_encode($data);

		}

		else {

			$this->setExtras();

			$json = (object) array(

				'extras' => $this->extras

			);

			if ($data !== FALSE) {

				$json->data = $data;

			}

			echo json_encode($json, RC_DEBUG_MODE ? 128 : 0);

		}

		exit();

	}

	// prepara alcuni URL per template e stack
	private function setURLs () {

		$this->defaultCatalogURL = RCUtilities::getURL(array(

			0 => 'catalog',
			1 => 'all',
			2 => '-',
			3 => RCUtilities::$settings->catalog->defaultSort,
			4 => 1

		), TRUE, FALSE);

		//$this->catalogURL = RCUtilities::loadSessionData('catalog_url', $this->defaultCatalogURL);
		$catalogURL = parse_url(RCUtilities::loadSessionData('catalog_url', $this->defaultCatalogURL));
		$this->catalogURL = RCUtilities::getURL(explode('/', $catalogURL['query']), TRUE, FALSE);

		$this->cartURL = RCUtilities::getURL(array(

			0 => 'cart',
			1 => 'overview'

		), TRUE);

	}

	// prepara gli extras per stack e temi
	private function setExtras () {

		$this->extras = (object) array(

			'cartURL'                => $this->cartURL,
			'catalogURL'             => $this->catalogURL,
			'cartItems'              => $this->cart->count,
			'cartItemsLocalized'     => (($this->cart->count == 0) ? RCUtilities::$localization->cart->cartWithNoItems : (($this->cart->count == 1) ? RCUtilities::$localization->cart->cartWithOneItem : sprintf(RCUtilities::$localization->cart->cartWithMultipleItems, $this->cart->count))),
			'cartTotal'              => $this->cart->subtotal + $this->cart->defaultTaxes,
			'cartTotalLocalized'     => RCUtilities::toCurrency($this->cart->subtotal + $this->cart->defaultTaxes, FALSE, FALSE),
			'cartItemsCSSClass'      => 'rc-extras-cart-items',
			'cartTotalCSSClass'      => 'rc-extras-cart-total'

		);

	}

	// carica la sezione in base all'URL
	public function getPage () {

		if (RCUtilities::$settings->general->offlineStore) {

			$this->page .= $this->loadTemplate('common/error', array(

				'title'      => RCUtilities::$localization->general->offlineTitle,
				'message'    => RCUtilities::$localization->general->offlineMessage,
				'showButton' => FALSE

			));

			return;

		}

		if (RCUtilities::isBlacklisted() !== TRUE) {

			$this->page .= $this->loadTemplate('common/error', array(

				'title'      => RCUtilities::$localization->blacklist->errorTitle,
				'message'    => RCUtilities::$localization->blacklist->errorMessageIP,
				'showButton' => FALSE

			));

			return;

		}

		if (empty(RCUtilities::$config->email->fromAddress)) {

			$this->page .= $this->loadTemplate('common/error', array(

				'title'      => 'Email Error',
				'message'    => 'Enter a valid email address in <code>Email</code> > <code>From</code> field in <i class="uk-icon-adjust"></i> Customization > <i class="uk-icon-cog"></i> Settings window.',
				'showButton' => FALSE

			));

			return;

		}

		$pars = RCUtilities::parseURL();

		$this->section = $pars[0];

		// mostra il disclaimer per l'accettazione dell'utilizzo dei cookies se impostato nelle preferenze e se l'utente non ha già accettato
		if (RCUtilities::$settings->general->showCookiesDisclaimer && !isset($_COOKIE['rc-cookies-disclaimer'])) {

			$this->page .= $this->loadTemplate('common/cookies');

		}

		switch ($pars[0]) {

			case 'cookies':

				setcookie('rc-cookies-disclaimer', 1, time() + 60 * 60 * 24 * 365, '/');

				RCUtilities::redirectToURL($_SERVER['HTTP_REFERER']);

			break;

			case 'extras':

				$this->sendJSON();

			break;

			case 'checklist':

				$this->showChecklist($pars);

			break;

			case 'referrer':

				$this->cart->setReferrer($pars[1]);

				RCUtilities::redirectToURL($this->catalogURL);

			break;

			case 'login':

				if (RCUtilities::$settings->dealers->loggedIn) {

					RCUtilities::redirectToURL($this->catalogURL);

				}

				else {

					$this->showLogin();

				}

			break;

			case 'catalog':

				$this->showCatalog($pars);

			break;

			case 'product':

				$this->showProduct($pars);

			break;

			case 'cart':

				$this->showCart($pars);

			break;

			case 'download':

				$this->showDownload($pars);

			break;

			case 'admin':

				$this->showAdmin($pars);

			break;

			case 'history':

				$this->showHistory($pars);

			break;

			case 'send-download':

				$this->sendDownload();

			break;

			default:

				RCUtilities::redirectToURL($this->catalogURL);

			break;

		}

		return $this->page;

	}

	// appende il box di debug al contenuto della pagina
	private function showDebug ($var) {

		$this->page .= $this->loadTemplate('common/debug', array(

			'code' => print_r($var, TRUE)

		));

	}

	// mostra la checklist
	private function showCheckList ($pars) {

		if (!isset($pars[1])) {

			$pars[1] = 'page';

		}

		switch ($pars[1]) {

			case 'page':

				$email = new RCOrder();

				$databaseVersion = '';

				if (RCUtilities::$settings->database->enable) {

					$database = new RCDatabase();

					$databaseVersion = $database->version;

				}

				$this->page .= $this->loadTemplate('common/checklist', array(

					'PHPMailerVersion' => \PHPMailer\SMTP::VERSION,
					'paypalEnabled'    => RCUtilities::$settings->payments->paypal->enable,
					'stripeVersion'    => RCUtilities::$settings->payments->stripe->enable ? \Stripe\Stripe::VERSION : '',
					'stripeEnabled'    => RCUtilities::$settings->payments->stripe->enable,
					'mollieVersion'    => RCUtilities::$settings->payments->mollie->enable ? Mollie_API_Client::CLIENT_VERSION : '',
					'mollieEnabled'    => RCUtilities::$settings->payments->mollie->enable,
					'databaseEnabled'  => RCUtilities::$settings->database->enable,
					'databaseVersion'  => $databaseVersion,
					'amazonEnabled'    => class_exists('S3')

				));

			break;

			case 'email':

				if (RCUtilities::$config->email->fromAddress == '') {

					$error  = 'No "From Address" set in Email settings.';

				}

				else {

					$email = new RCOrder();

					$email->checkSMTP();

					$error = $email->error;

				}

				$this->sendJSON($error);

			break;

			case 'paypal':

				$payment = new RCPayment('paypal');

				$payment->check();

				$this->sendJSON($payment->error);

			break;

			case 'stripe':

				$payment = new RCPayment('stripe');

				$payment->check();

				$this->sendJSON($payment->error);

			break;

			case 'square':

				$payment = new RCPayment('square');

				$payment->check();

				$this->sendJSON($payment->error);

			break;

			case 'mollie':

				$payment = new RCPayment('mollie');

				$payment->check();

				$this->sendJSON($payment->error);

			break;

			case 'database':

				$database = new RCDatabase();

				$this->sendJSON($database->error);

			break;

			case 'amazon':

				if (!defined('CURL_SSLVERSION_TLSv1')) {

					define('CURL_SSLVERSION_TLSv1', 1);

				}

				S3::setAuth(RCUtilities::$config->amazon->accessKey, RCUtilities::$config->amazon->secretKey);

				$rest = new S3Request('GET');
				$response = $rest->getResponse();

				$this->sendJSON(isset($response->error['message']) ? $response->error['message'] : '');

			break;

		}

	}

	// mostra la pagina di autenticazione
	private function showLogin () {

		$this->page .= $this->loadTemplate('common/login');

	}

	// mostra la pagina del catalogo
	private function showCatalog ($pars) {

		// se l'URL non contiene almeno 4 parametri, rimanda al catalogo di default
		if (count($pars) < 5) {

			RCUtilities::redirectToURL($this->defaultCatalogURL);

		}

		$this->catalog->applyFilter($pars[1], strtolower($pars[2]));
		$this->catalog->sortProducts($pars[3]);

		if (RC_DEBUG_MODE) {

			$this->showDebug($this->catalog);

		}

		$products = '';

		foreach ($this->catalog->getProductsPage($pars[4]) as $i => $product) {

			$products .= $this->loadTemplate('catalog/' . RCUtilities::$settings->catalog->viewMode, array(

				'product'  => $product

			));

		}

		$this->page .= $this->loadTemplate('catalog/page', array(

			'page'     => $pars[4],
			'filter'   => $pars[1],
			'search'   => (($pars[1] == 'search') ? $pars[2] : ''),
			'cart'     => $this->loadTemplate('common/cart', array(

				'fullWidth'  => TRUE,
				'forceSmall' => FALSE

			)),
			'products' => $products

		));

		//$this->catalogURL = RCUtilities::saveSessionData('catalog_url', RCUtilities::getURL(array(), TRUE, FALSE));
		$this->catalogURL = RCUtilities::saveSessionData('catalog_url', RCUtilities::getURL(array(

			0 => 'catalog',
			1 => $pars[1],
			2 => $pars[2],
			3 => $pars[3],
			4 => $pars[4]

		), TRUE, FALSE));

	}

	// mostra la pagina del prodotto
	private function showProduct ($pars) {

		// se l'URL non contiene almeno 2 parametri, rimanda al catalogo di default
		if (count($pars) < 2) {

			RCUtilities::redirectToURL($this->defaultCatalogURL);

		}

		switch ($pars[1]) {

			case 'page':

				$product = $this->catalog->getProduct(isset($pars[2]) ? $pars[2] : NULL);

				// se non trova un prodotto corrispondente mostra la pagina di errore
				if ($product == FALSE) {

					$this->page .= $this->loadTemplate('common/error', array(

						'title'      => RCUtilities::$localization->general->errorCode,
						'message'    => RCUtilities::$localization->general->pageNotFound,
						'showButton' => TRUE,
						'buttonURL'  => $this->catalogURL,
						'buttonText' => RCUtilities::$localization->general->goBack

					));

					return;

				}

				if ($product->hasCustomPage) {

					RCUtilities::redirectToURL($product->link);

				}

				if (RC_DEBUG_MODE) {

					$this->showDebug($product);

				}

				$related = '';

				foreach ($product->related as $relatedProduct) {

					$related .= $this->loadTemplate('product/related', array(

						'product' => $relatedProduct

					));

				}

				$this->header .= $this->loadTemplate('product/metadata', array(

					'name'        => $product->name,
					'description' => $product->description->short,
					'images'      => $product->images

				));

				$this->page .= $this->loadTemplate('product/page', array(

					'product'  => $product,
					'cart'     => $this->loadTemplate('common/cart', array(

						'fullWidth'  => FALSE,
						'forceSmall' => TRUE

					)),
					'social'   => $this->loadTemplate('product/social', array(

						'name'  => $product->name,
						'image' => $product->images[0]->thumbnailPath

					)),
					'tags'     => $this->loadTemplate('product/tags', array(

						'tags'       => $product->tags,
						'categories' => $product->categories

					)),
					'rating'   => $this->loadTemplate('product/rating', array(

						'average' => $product->ratingAverage,
						'count'   => $product->ratingCount

					)),
					'gallery'  => $this->loadTemplate('product/gallery', array(

						'images'  => $product->images,
						'product' => addslashes($product->name)

					)),
					'versions' => $this->loadTemplate('product/versions', array(

						'ID'              => $product->ID,
						'name'            => $product->name,
						'versions'        => $product->versions,
						'options'         => $product->options,
						'requiredOptions' => $product->requiredOptions

					)),
					'related'  => $related

				));

			break;

			case 'set-options';

				$options = (object) $_POST['rc-product'];

				$product = $this->catalog->getProduct($options->ID);

				$json = $product->setVersionAndOptions($options);

				$json->price = RCUtilities::toCurrency(($json->price + $json->price * RCUtilities::$settings->product->defaultTaxes[$json->taxesTier] / 100) * $json->selectedQuantity);

				$this->sendJSON($json);

			break;

			default:

				RCUtilities::redirectToURL($this->defaultCatalogURL);

			break;

		}

	}

	// mostra la pagina del carrello
	private function showCart ($pars) {

		// se l'URL non contiene almeno 1 parametro, rimanda all'overview del carrello
		if (count($pars) < 2) {

			RCUtilities::redirectToURL($this->cartURL);

		}

		if (RC_DEBUG_MODE) {

			$this->showDebug($this->cart);

		}

		if (!empty($this->database->error)) {

			$this->page .= $this->loadTemplate('common/error', array(

				'title'      => 'Database Error',
				'message'    => $this->database->error,
				'showButton' => TRUE,
				'buttonURL'  => $this->catalogURL,
				'buttonText' => RCUtilities::$localization->general->goBack

			));

			return;

		}

		$this->page .= $this->loadTemplate('cart/header', array(

			'sections' => $this->cart->getSections($pars[1])

		));

		switch ($pars[1]) {

			case 'overview':

				$this->page .= $this->loadTemplate(empty($this->cart->items) ? 'cart/empty' : 'cart/overview');

			break;

			case 'update':

				foreach ($_POST['rc-cart-item'] as $item) {

					if (isset($item['remove'])) {

						$this->cart->removeItem($item['ID']);

					}

					else {

						$this->cart->updateQuantity($item['ID'], $item['quantity']);

					}


				}

			break;

			case 'add-item':

				$count = $this->cart->count;

				if (isset($_POST['rc-product'])) {

					if (($product = $this->catalog->getProduct($_POST['rc-product']['ID'])) !== FALSE) {

						$item = $product->setVersionAndOptions((object) $_POST['rc-product']);

						if ($item !== FALSE) {

							$this->cart->addItem($item);

						}

					}

				}

				$added = $count != $this->cart->count;

				$json = (object) array(

					'message' => $this->loadTemplate('common/notify', array(

						'icon'    => $added ? 'check' : 'warning',
						'message' => $added ? RCUtilities::$localization->cart->productAdded : RCUtilities::$localization->cart->productNotAdded

					)),
					'type'    => $added ? 'success' : 'warning'

				);

				$this->sendJSON($json);

			break;

			case 'apply-coupon-code':

				$this->cart->applyCouponCode($_POST['couponCode']);

				$this->sendJSON((object) array(

					'valid'   => $this->cart->coupon !== FALSE,
					'message' => RCUtilities::$localization->cart->couponCodeInvalid

				));

			break;

			case 'remove-coupon-code':

				$this->cart->removeCouponCode();

				RCUtilities::redirectToURL(RCUtilities::getURL(array(

					0 => 'cart',
					1 => 'overview'

				), TRUE));

			break;

			case 'address':

				if (empty($this->cart->items)) {

					RCUtilities::redirectToURL(RCUtilities::getURL(array(

						0 => 'cart',
						1 => 'overview'

					), TRUE));

				}

				$this->page .= $this->loadTemplate('cart/address');

			break;

			case 'set-address':

				$required = $this->cart->validateCustomerDetails($_POST);

				$messages = array();

				if (RCUtilities::isBlacklisted($_POST['rc-cart-customer-email-address']) !== TRUE) {

					$messages[] = $this->loadTemplate('common/notify', array(

						'icon'    => 'warning',
						'message' => RCUtilities::$localization->blacklist->errorMessageEmail

					));

				}

				if (!empty($required)) {

					$messages[] = $this->loadTemplate('common/notify', array(

						'icon'    => 'warning',
						'message' => RCUtilities::$localization->cart->fieldsRequired

					));

				}

				if (in_array('rc-cart-customer-vat-id', $required)) {

					$messages[] = $this->loadTemplate('common/notify', array(

						'icon'    => 'warning',
						'message' => RCUtilities::$localization->cart->invalidVATID

					));

				}

				$json = (object) array(

					'required' => $required,
					'messages' => $messages

				);

				$this->sendJSON($json);

			break;

			case 'options':

				if ($this->cart->customer->valid) {

					$this->page .= $this->loadTemplate('cart/options');

				}

				else {

					RCUtilities::redirectToURL(RCUtilities::getURL(array(

						0 => 'cart',
						1 => 'address'

					), TRUE));

				}

			break;

			case 'set-options':

				$json = (object) array(

					'required' => $this->cart->validateOptions($_POST['rc-cart']['options']),
					'message'  => $this->loadTemplate('common/notify', array(

						'icon'    => 'warning',
						'message' => RCUtilities::$localization->cart->fieldsRequired

					))

				);

				$this->sendJSON($json);

			break;

			case 'shipping':

				if ($this->cart->shippable && ((!empty($this->cart->options->set) && $this->cart->options->valid) || (empty($this->cart->options->set) && $this->cart->customer->valid))) {

					$this->page .= $this->loadTemplate('cart/shipping', array(

						'methods' => $this->cart->getShippingMethods()

					));

				}

				else {

					RCUtilities::redirectToURL(RCUtilities::getURL(array(

						0 => 'cart',
						1 => empty($this->cart->options->set) ? 'address' : 'options'

					), TRUE));

				}

			break;

			case 'set-shipping':

				$json = (object) array(

					'valid'   => $this->cart->validateShippingMethod($_POST),
					'message' => $this->loadTemplate('common/notify', array(

						'icon'    => 'warning',
						'message' => RCUtilities::$localization->cart->selectShippingMethod

					))

				);

				$this->sendJSON($json);

			break;

			case 'payment':

				if ((!$this->cart->shippable && $this->cart->customer->valid) || $this->cart->shipping->valid) {

					$methods = $this->cart->getPaymentMethods();

					$showCreditCard = FALSE;

					foreach ($methods as $method) {

						if ($method->selected && $method->requireCreditCard) {

							$showCreditCard = TRUE;

						}

					}

					$this->page .= $this->loadTemplate('cart/payment', array(

						'methods'        => $methods,
						'showCreditCard' => $showCreditCard

					));

				}

				else {

					RCUtilities::redirectToURL(RCUtilities::getURL(array(

						0 => 'cart',
						1 => 'shipping'

					), TRUE));

				}

			break;

			case 'set-payment':

				$paymentID = $_POST['rc-cart-payment'];

				$json = (object) array(

					'paymentValid'    => FALSE,
					'creditCardValid' => !RCUtilities::$settings->payments->{$paymentID}->requireCreditCard,
					'error'    => $this->loadTemplate('common/notify', array(

						'icon'    => 'warning',
						'message' => RCUtilities::$localization->cart->selectPaymentMethod

					))

				);

				$payment = new RCPayment($paymentID);

				$options = NULL;

				if (RCUtilities::$settings->payments->{$paymentID}->requireCreditCard) {

					$options = (object) array(

						'customer'            => $this->cart->customer,
						'cardHolderFirstName' => $_POST['rc-cart-credit-card-holder-first-name'],
						'cardHolderLastName'  => $_POST['rc-cart-credit-card-holder-last-name'],
						'cardNumber'          => $_POST['rc-cart-credit-card-number'],
						'cardExpMonth'        => $_POST['rc-cart-credit-card-expiration-month'],
						'cardExpYear'         => $_POST['rc-cart-credit-card-expiration-year'],
						'cardCVC'             => $_POST['rc-cart-credit-card-cvc']

					);

				}

				if ($payment->authorize($options)) {

					$this->cart->setPaymentMethod($paymentID);

					$json->paymentValid = TRUE;
					$json->creditCardValid = TRUE;

					if (RCUtilities::$settings->payments->{$paymentID}->requireCreditCard) {

						$this->cart->setCreditCardNumber($_POST['rc-cart-credit-card-number']);

					}

				}

				else {

					$json->creditCardError = $this->loadTemplate('common/notify', array(

						'icon'    => 'warning',
						'message' => $payment->error

					));

				}

				$this->sendJSON($json);

			break;

			case 'review':

				if ($this->cart->payment->valid) {

					if ($this->cart->payment->method->type == 'paypal' && !(isset($_GET['paymentId']) && isset($_GET['PayerID']))) {

						$payment = new RCPayment('paypal');

						$redirectURL = $payment->prepare($this->cart);

						if ($redirectURL !== FALSE) {

							$this->page .= $this->loadTemplate('cart/redirect', array(

								'redirectURL' => $redirectURL

							));

						}

						else {

							$this->showDebug($payment->error);

						}

					}

					else {

						$this->page .= $this->loadTemplate('cart/review');

					}

				}

				else {

					RCUtilities::redirectToURL(RCUtilities::getURL(array(

						0 => 'cart',
						1 => 'payment'

					), TRUE));

				}

			break;

			case 'checkout':

				if (empty($this->cart->items)) {

					RCUtilities::redirectToURL(RCUtilities::getURL(array(

						0 => 'cart',
						1 => 'overview'

					), TRUE));

				}

				$payment = new RCPayment($this->cart->payment->method->type);

				$options = NULL;

				switch ($this->cart->payment->method->type) {

					case 'paypal':

						$options = (object) array(

							'paymentID' => $pars[2],
							'payerID'   => $pars[3],

						);

					break;

					case 'stripe':
					case 'mollie':

						$options = (object) array(

							'cart' => $this->cart

						);

					break;

				}

				if ($this->cart->payment->method->type == 'stripe') {
					
					if ($pars[2] != 'complete') {

						$transactionID = $payment->execute($options);

						exit($this->loadTemplate('cart/stripe', array(

							'publicKey' => RCUtilities::$config->stripe->publicKey,
							'sessionID' => $transactionID

						)));

					} else {

						$transactionID = '';

					}

				} else {

					$transactionID = $payment->execute($options);

				}

				if ($transactionID !== FALSE) {

					$this->cart->saveTransactionID($transactionID);

					$order = new RCOrder();

					$order->setSubject(str_replace('%ORDER_ID%', $this->cart->ID, RCUtilities::$localization->order->title));
					$order->setAddress($this->cart->customer->emailAddress, $this->cart->customer->name);

					if (!isset(RCUtilities::$settings->digitalDelivery->disableDeliveryForOfflinePayments) || !RCUtilities::$settings->digitalDelivery->disableDeliveryForOfflinePayments || ($this->cart->payment->method->type != 'order' && $this->cart->payment->method->type != 'cash')) {

						if ($this->databaseEnabled) {

							$this->database->addDownloads($this->cart);

						}

						else {

							$this->cart->generatesDownloadLinks();

						}

					}

					$order->setBodies($this->getReceipt(FALSE), $this->getReceipt(TRUE));

					$result = $order->send();

					if ($result !== TRUE) {

						$this->page .= $this->loadTemplate('cart/error', array(

							'error' => $result

						));

					}

					else {

						if (RCUtilities::$config->general->webhook != '') {

							$webhook = RCUtilities::sendWebhookData(RCUtilities::$config->general->webhook, $this->cart);

							if (!empty($webhook->error)) {

								$this->showDebug('Webhook: ' . RCUtilities::$config->general->webhook . PHP_EOL . PHP_EOL . print_r($webhook, TRUE));

							}

						}

						if ($this->databaseEnabled) {

							$this->database->updateProducts($this->catalog, $this->cart);

							if ($this->cart->coupon !== FALSE) {

								$this->database->updateCoupons($this->cart->coupon);

							}

							if (isset(RCUtilities::$settings->database->saveOrderDetails) && RCUtilities::$settings->database->saveOrderDetails) {

								$this->database->addOrder($this->cart);

							}

						}

						if (RCUtilities::$settings->admin->getPushNotifications) {

							RCUtilities::sendPushNotification($this->cart->total, $this->cart->customer->name, $this->cart->ID);

						}

						$this->cart->reset();

						if (isset(RCUtilities::$settings->cart->thankYouPage) && RCUtilities::$settings->cart->thankYouPage == 'custom' && isset(RCUtilities::$settings->cart->customThankYouPageURL) && !empty(RCUtilities::$settings->cart->customThankYouPageURL)) {

							RCUtilities::redirectToURL(RCUtilities::$settings->cart->customThankYouPageURL);

						}

						else {

							RCUtilities::redirectToURL(RCUtilities::getURL(array(

								0 => 'cart',
								1 => 'success'

							), TRUE));

						}

					}

				}

				else {

					$this->page .= $this->loadTemplate('cart/error', array(

						'error' => $payment->error

					));

				}

			break;

			case 'confirm':

				if ($this->cart->payment->method->type == 'mollie') {

					$mollie = new RCMollie();

					if ($mollie->getPayment($this->cart)) {

						$this->cart->saveTransactionID($mollie->transactionID);

					}

					else {

						RCUtilities::redirectToURL(RCUtilities::getURL(array(

							0 => 'cart',
							1 => 'review'

						), TRUE));

					}

				}

			break;

			case 'success':

				$this->page .= $this->loadTemplate('cart/checkout');

			break;

			case 'receipt':

				if (ob_get_contents()) {

					ob_end_clean();

				}

				echo $this->getReceipt(FALSE);

				exit();

			break;

			default:

				RCUtilities::redirectToURL($this->extras->cartURL);

			break;

		}

	}

	// se l'email del cliente non è in blacklist visualizza la pagina di download del prodotto
	private function showDownload ($pars) {

		if (count($pars) < 2) {

			RCUtilities::redirectToURL($this->catalogURL);

		}

		$error = FALSE;

		if ($this->databaseEnabled) {

			$data = $this->database->getDownload($pars[1], $pars[2]);

		}

		else if (isset($pars[3])){

			$data = (object) array(

				'time'  => RCUtilities::dataEncrypt(base64_decode($pars[1])),
				'SKU'   => RCUtilities::dataEncrypt(base64_decode($pars[2])),
				'email' => RCUtilities::dataEncrypt(base64_decode($pars[3]))

			);

		}

		else {

			$data = FALSE;

		}

		if ($data === FALSE) {

			$error = '';

		}

		else if (RCUtilities::isBlacklisted($data->email) !== TRUE) {

			$error = RCUtilities::$localization->blacklist->errorMessageEmail;

		}

		else if (time() > $data->time + RCUtilities::$settings->digitalDelivery->expiration * 60 * 60 || (RCUtilities::$settings->digitalDelivery->downloadLimit > 0 && isset($data->downloads) && $data->downloads > 0 && $data->downloads >= RCUtilities::$settings->digitalDelivery->downloadLimit)) {

			$error = RCUtilities::$localization->download->linkExpired;

		}

		else {

			$product = $this->catalog->getDownloadLinks($data->SKU);

			if ($product === FALSE) {

				$error = RCUtilities::$localization->download->productNotFound;

			}

		}

		if ($error !== FALSE) {

			$this->page .= $this->loadTemplate('common/error', array(

				'title'      => RCUtilities::$localization->download->errorTitle,
				'message'    => $error,
				'showButton' => TRUE,
				'buttonURL'  => $this->catalogURL,
				'buttonText' => RCUtilities::$localization->general->goBack

			));

			return;

		}

		if ($this->databaseEnabled) {

			$this->database->updateDownload($pars[1], $pars[2]);

		}

		$this->page .= $this->loadTemplate('download/page', array(

			'product' => $product

		));

	}

	// mostra la pagina degli ordini
	private function showHistory ($pars) {

		if (count($pars) < 2) {

			RCUtilities::redirectToURL($this->catalogURL);

		}

		switch ($pars[1]) {

			case 'order':

				$this->page .= $this->getHistoryOrder($pars[2]);

			break;

			case 'rating':

				$this->page .= $this->getHistoryRating($pars[2]);

			break;

			case 'rate-product':

				print_r($_POST);

				$this->database->setRating($_POST['order'], $_POST['product'], $_POST['rate']);

			break;

		}

	}

	// mostra la pagina dell'ordine
	private function getHistoryOrder ($ID) {

		$order = $this->database->getOrder($ID, FALSE);

		if (RC_DEBUG_MODE) {

			$this->showDebug($order);

		}

		return $this->loadTemplate('history/order', array(

			'ID'    => $ID,
			'order' => $order

		));

	}

	// mostra la pagina delle valutazioni
	private function getHistoryRating ($ID) {

		$order = $this->database->getOrder($ID, FALSE);

		$products = array();

		if ($order !== FALSE) {

			foreach ($order->Items as $item) {

				if (isset($item->product->productID)) {

					$products[$item->product->productID] = $item->product;

				}

			}

		}

		if (RC_DEBUG_MODE) {

			$this->showDebug($order);

		}

		return $this->loadTemplate('history/rating', array(

			'order'    => $order,
			'products' => $products

		));

	}

	// mostra la pagina dell'amministrazione
	private function showAdmin ($pars) {

		$style = (RCUtilities::$settings->general->style != 'flat') ? ('.' . RCUtilities::$settings->general->style) : '';

		$authError = FALSE;

		if (isset($_POST['username']) && isset($_POST['password'])) {

			$authError = TRUE;

			if (!empty(RCUtilities::$config->admin->usernameWrite) && $_POST['username'] == RCUtilities::$config->admin->usernameWrite && !empty(RCUtilities::$config->admin->passwordWrite) && md5($_POST['password']) == RCUtilities::$config->admin->passwordWrite) {

				$authType = 'write';
				$_SESSION['rc-admin-auth'] = md5(RCUtilities::$config->admin->usernameWrite . md5(RCUtilities::$config->admin->passwordWrite));

			}

			else if (!empty(RCUtilities::$config->admin->usernameRead) && $_POST['username'] == RCUtilities::$config->admin->usernameRead && !empty(RCUtilities::$config->admin->passwordRead) && md5($_POST['password']) == RCUtilities::$config->admin->passwordRead) {

				$authType = 'read';
				$_SESSION['rc-admin-auth'] = md5(RCUtilities::$config->admin->usernameRead . md5(RCUtilities::$config->admin->passwordRead));

			}

		}

		else if (isset($_SESSION['rc-admin-auth'])) {

			if ($_SESSION['rc-admin-auth'] === md5(RCUtilities::$config->admin->usernameWrite . md5(RCUtilities::$config->admin->passwordWrite))) {

				$authType = 'write';

			}

			else if ($_SESSION['rc-admin-auth'] === md5(RCUtilities::$config->admin->usernameRead . md5(RCUtilities::$config->admin->passwordRead))) {

				$authType = 'read';

			}

			else {

				unset($_SESSION['rc-admin-auth']);

			}

		}

		if (count($pars) < 2) {

			RCUtilities::redirectToURL(RCUtilities::getURL(array(

				0 => 'admin',
				1 => (RCUtilities::$settings->admin->firstPage == 'stats') ? 'stats': 'orders'

			), TRUE));

		}

		if (!isset($_SESSION['rc-admin-auth'])) {

			$error = '';

			if (empty(RCUtilities::$config->admin->usernameWrite) || empty(RCUtilities::$config->admin->passwordWrite)) {

				$error = 'credentials';

			}

			else if (!$this->databaseEnabled) {

				$error = 'database';

			}

			if (ob_get_contents()) {

				ob_end_clean();

			}

			echo $this->loadTemplate('admin/login', array(

				'error'     => $error,
				'style'     => $style,
				'authError' => $authError

			));

			exit();

		}

		switch ($pars[1]) {

			case 'search':

				$this->sendJSON($this->getAdminOrdersJSON($_POST['search']), TRUE);

			break;

			case 'orders':

				$title = RCUtilities::$localization->admin->toolbarOrders;
				$body = $this->getAdminOrders($pars);

			break;

			case 'order';

				$title = RCUtilities::$localization->admin->toolbarOrder . ' ' . $pars[2];
				$body = $this->getAdminOrder($pars[2], $authType);

			break;

			case 'products':

				$title = RCUtilities::$localization->admin->toolbarProducts;
				$body = $this->getAdminProducts($pars, $authType);

			break;

			case 'coupons':

				$title = RCUtilities::$localization->admin->toolbarCoupons;
				$body = $this->getAdminCoupons($authType);

			break;

			case 'downloads':

				$title = RCUtilities::$localization->admin->toolbarDownloads;
				$body = '';

			break;

			case 'ratings':

				$title = RCUtilities::$localization->admin->toolbarRatings;
				$body = $this->getAdminRatings($authType);

			break;

			case 'stats':

				$title = RCUtilities::$localization->admin->toolbarStats;
				$body = $this->getAdminStats();

			break;

			case 'logout':

				unset($_SESSION['rc-admin-auth']);

				RCUtilities::redirectToURL(RCUtilities::getURL(array(

					0 => 'admin',
					1 => (RCUtilities::$settings->admin->firstPage == 'stats') ? 'stats' : 'orders'

				), TRUE));

			break;

			default:

				RCUtilities::redirectToURL(RCUtilities::getURL(array(

					0 => 'admin',
					1 => (RCUtilities::$settings->admin->firstPage == 'stats') ? 'stats' : 'orders'

				), TRUE));

			break;

		}

		$lowProductsCount = 0;

		foreach ($this->catalog->products as $product) {

			foreach ($product->versions as $version) {

				if (isset($version->enableStockManagement) && $version->enableStockManagement && $version->stockQuantity <= 5) {

					$lowProductsCount++;

				}

			}

		}

		if (ob_get_contents()) {

			ob_end_clean();

		}

		echo $this->loadTemplate('admin/page', array(

			'style'              => $style,
			'pendingOrdersCount' => $this->database->getPendingOrdersCount(),
			'lowProductsCount'   => $lowProductsCount,
			'title'              => $title,
			'section'            => $pars[1],
			'body'               => $body

		));

		exit();

	}

	// mostra l'admin degli ordini
	private function getAdminOrders ($pars) {

		$ordersSearch = isset($pars[5]) && $pars[5] != '*' ? $pars[5] : '';
		$ordersStatus = isset($pars[2]) ? $pars[2] : 'all';
		$ordersFromDate = isset($pars[3]) && $pars[3] != '*' ? date('Y-m-d', strtotime($pars[3])) : '';
		$ordersToDate = isset($pars[4]) && $pars[4] != '*' ? date('Y-m-d', strtotime($pars[4])) : '';
		$ordersStart = (isset($pars[6]) && $pars[6] > 0) ? (int) $pars[6] : 1;

		$grouped = count($pars) < 3;

		$data = $this->database->getOrders($ordersSearch, $ordersStatus, $ordersFromDate, $ordersToDate, $ordersStart - 1, 20);

		$rawOrders = $data->orders;

		$orders = (object) array(

			'today'     => (object) array(

				'title' => 'Today',
				'list'  => array(),
				'count' => $data->ordersToday,
				'total' => $data->totalToday

			),
			'yesterday' => (object) array(

				'title' => 'Yesterday',
				'list'  => array(),
				'count' => $data->ordersYesterday,
				'total' => $data->totalYesterday

			),
			'month'     => (object) array(

				'title' => 'This Month',
				'list'  => array(),
				'count' => $data->ordersMonth,
				'total' => $data->totalMonth

			),
			'year'      => (object) array(

				'title' => 'This Year',
				'list'  => array(),
				'count' => $data->ordersYear,
				'total' => $data->totalYear

			),
			'all'       => (object) array(

				'title' => 'Orders',
				'list'  => array(),
				'count' => $data->count,
				'total' => $data->total

			)

		);

		foreach ($rawOrders as $rawOrder) {

			$order = (object) array(

				'ID'     => $rawOrder->ID,
				'date'   => strtotime($rawOrder->Date . ' UTC'),
				'status' => $rawOrder->Status,
				'name'   => $rawOrder->CustomerName,
				'total'  => $rawOrder->Total,
				'refund' => $rawOrder->TotalRefunded

			);

			$total = ($order->status == 'cancelled') ? 0 : ($order->total - $order->refund);

			if ($grouped && date('Y-m-d') == date('Y-m-d', $order->date)) {

				$order->date = date('H:i', $order->date);
				$orders->today->list[] = $order;

			}

			else if ($grouped && date('Y-m-d', strtotime('yesterday')) == date('Y-m-d', $order->date)) {

				$order->date = date('H:i', $order->date);
				$orders->yesterday->list[] = $order;

			}

			else if ($grouped && date('Y-m') == date('Y-m', $order->date)) {

				$order->date = date('Y-m-d H:i', $order->date);
				$orders->month->list[] = $order;

			}

			else if ($grouped && date('Y') == date('Y', $order->date)) {

				$order->date = date('Y-m-d H:i', $order->date);
				$orders->year->list[] = $order;

			}

			else {

				$order->date = date('Y-m-d H:i', $order->date);
				$orders->all->list[] = $order;

			}

		}

		return $this->loadTemplate('admin/orders', array(

			'count'          => $data->count,
			'page'           => $ordersStart,
			'ordersSearch'   => $ordersSearch,
			'ordersStatus'   => $ordersStatus,
			'ordersFromDate' => $ordersFromDate,
			'ordersToDate'   => $ordersToDate,
			'orders'         => $orders

		));

	}

	// ritorna la lista degli ordini per l'autocompletamento
	private function getAdminOrdersJSON ($search) {

		$data = $this->database->getOrders($search, '', '', '', 0, 10);

		$orders = array();

		foreach ($data->orders as $rawOrder) {

			$orders[] = (object) array(

				'name' => $rawOrder->CustomerName,
				'url'  => RCUtilities::getURL(array(

					0 => 'admin',
					1 => 'order',
					2 => $rawOrder->ID

				), TRUE),
				'date' => date('Y-m-d H:i', strtotime($rawOrder->Date)),
				'id'   => $rawOrder->ID

			);

		}

		return $orders;

	}

	// mostra l'ordine in amministrazione
	private function getAdminOrder ($ID, $authType) {

		if (isset($_POST['ID']) && $authType == 'write') {

			$this->database->updateOrder((object) $_POST);

		}

		$order = $this->database->getOrder($ID);

		return $this->loadTemplate('admin/order', array(

			'order'    => $order,
			'authType' => $authType

		));

	}

	// mostra le stats
	private function getAdminStats () {

		$stats = $this->database->getStats();

		$stats->trendsTodayPercentage = ($stats->trendsToday[1]->Total == 0) ? ($stats->trendsToday[0]->Total == 0 ? 0 : 100) : ($stats->trendsToday[0]->Total * 100 / $stats->trendsToday[1]->Total - 100);

		$max = 0;

		foreach ($stats->trendsToday as $day) {

			$max = max($max, $day->Total);

		}

		foreach ($stats->trendsToday as $day) {

			$day->Total = (($max == 0 ) ? 0 : ($day->Total * 100 / $max)) + 50;

		}

		$stats->trendsThisMonthPercentage = ($stats->trendsThisMonth[1]->Total == 0) ? ($stats->trendsThisMonth[0]->Total == 0 ? 0 : 100) : ($stats->trendsThisMonth[0]->Total * 100 / $stats->trendsThisMonth[1]->Total - 100);

		$max = 0;

		foreach ($stats->trendsThisMonth as $month) {

			$max = max($max, $month->Total);

		}

		foreach ($stats->trendsThisMonth as $month) {

			$month->Total = (($max == 0 ) ? 0 : ($month->Total * 100 / $max)) + 50;

		}

		$stats->trendsThisYearPercentage = ($stats->trendsThisYear[1]->Total == 0) ? ($stats->trendsThisYear[0]->Total == 0 ? 0 : 100) : ($stats->trendsThisYear[0]->Total * 100 / $stats->trendsThisYear[1]->Total - 100);

		$max = 0;

		$stats->trendsThisYearHide = TRUE;

		foreach ($stats->trendsThisYear as $i => $year) {

			if ($i > 0 && $year->Orders > 0) {

				$stats->trendsThisYearHide = FALSE;

			}

			$max = max($max, $year->Total);

		}

		foreach ($stats->trendsThisYear as $year) {

			$year->Total = (($max == 0 ) ? 0 : ($year->Total * 100 / $max)) + 50;

		}

		$maxTotal = 0;

		foreach ($stats->sales as $sale) {

			$maxTotal = max($maxTotal, $sale->Total);

		}

		foreach ($stats->sales as $sale) {

			$sale->Total = ($max == 0 ) ? 0 : ($sale->Total * 100 / $maxTotal);

		}

		return $this->loadTemplate('admin/stats', array(

			'stats' => $stats

		));

	}

	// mostra i prodotti
	private function getAdminProducts ($pars, $authType) {

		if (isset($_POST['rc-admin-products']) && $authType == 'write') {

			$this->database->updateAdminProducts($_POST['rc-admin-products']);

			if (ob_get_contents()) {

				ob_end_clean();

			}

			header('Refresh: 0');

			exit();

		}

		if (count($pars) < 5) {

			$search = '';
			$sort = 'quantity';
			$page = '1';

		}

		else {

			$search = ($pars[2] == '*') ? '' : $pars[2];
			$sort = $pars[3];
			$page = $pars[4];

		}

		$products = array();

		foreach ($this->catalog->products as $product) {

			foreach ($product->versions as $version) {

				if (isset($version->enableStockManagement) && $version->enableStockManagement) {

					if (!empty($search) && (stripos($product->name, $search) === FALSE) && (stripos($version->name, $search) === FALSE) && (stripos($version->SKU, $search) === FALSE)) {

						continue;

					}

					$products[] = (object) array(

						'name'      => $product->name,
						'version'   => $version->name,
						'SKU'       => $version->SKU,
						'available' => $version->available,
						//'stock'     => isset($version->enableStockManagement) && $version->enableStockManagement,
						//'quantity'  => isset($version->enableStockManagement) ? $version->stockQuantity : 0
						'quantity'  => $version->stockQuantity

					);

				}

			}

		}

		if ($sort == 'name') {

			usort($products, function ($a, $b) {

				return strcmp(strtolower($a->name), strtolower($b->name));

			});

		}

		else {

			usort($products, function ($a, $b) {

				/*if (!$a->stock) {

					return 1;

				}

				if (!$b->stock) {

					return -1;

				}*/

				return $a->quantity - $b->quantity;

			});

		}

		return $this->loadTemplate('admin/products', array(

			'count'    => count($products),
			'search'   => $search,
			'sort'     => $sort,
			'products' => array_slice($products, 20 * ($page - 1), 20),
			'page'     => $page,
			'authType' => $authType

		));

	}

	// mostra i coupon
	private function getAdminCoupons ($authType) {

		if (isset($_POST['rc-admin-coupons']) && $authType == 'write') {

			$this->database->updateAdminCoupons($_POST['rc-admin-coupons']);

			if (ob_get_contents()) {

				ob_end_clean();

			}

			header('Refresh: 0');

			exit();

		}

		$coupons = array();

		if (RCUtilities::$settings->cart->acceptCouponCodes) {

			foreach (RCUtilities::$settings->cart->costs as $cost) {

				if (
					isset($cost->couponRequired) &&
					$cost->couponRequired &&
					$cost->hasLimit
				) {

					$coupons[] = $cost;

				}

			}

		}

		return $this->loadTemplate('admin/coupons', array(

			'coupons'  => $coupons,
			'authType' => $authType

		));

	}

	// mostra le recensioni
	private function getAdminRatings ($authType) {

		if (isset($_POST['rc-admin-ratings']) && $authType == 'write') {

			$this->database->updateAdminRatings($_POST['rc-admin-ratings']);

			if (ob_get_contents()) {

				ob_end_clean();

			}

			header('Refresh: 0');

			exit();

		}

		$ratings = array();

		return $this->loadTemplate('admin/ratings', array(

			'ratings'  => $ratings,
			'authType' => $authType

		));

	}

	// restituisce il codice richiesto dallo stack
	public function getStack ($stack, $options = array()) {

		$result = (object) array(

			'header'   => $this->header,
			'metadata' => '',
			'page'     => ''

		);

		switch ($stack) {

			case 'cart':

				$result->page = $this->loadTemplate('common/cart', array(

					'fullWidth'  => TRUE,
					'forceSmall' => FALSE

				));

			break;

			case 'grid':

				$this->catalog->resetProductsList();
				$this->catalog->applyFilter($options->filterType, strtolower(urldecode($options->filterValue)));
				$this->catalog->sortProducts($options->sort);

				$products = '';

				foreach ($this->catalog->products as $i => $product) {

					if ($i == $options->products) {

						break;

					}

					$products .= $this->loadTemplate('catalog/box', array(

						'product' => $product

					));

				}

				$result->page = $this->loadTemplate('common/grid', array(

					'products' => $products,
					'columns'  => $options->columns

				));

			break;

			case 'product':

				$product = $this->catalog->getProduct(($options->showProduct != 'random') ? $options->productID : NULL);

				if ($product !== FALSE) {

					switch ($options->part) {

						case 'social':

							$result->metadata = $this->loadTemplate('product/metadata', array(

								'name'        => $product->name,
								'description' => $product->description->short,
								'images'      => $product->images

							));

							$result->page = $this->loadTemplate('product/social', array(

								'name'  => $product->name,
								'image' => $product->images[0]->thumbnailPath

							));

						break;

						case 'tags':

							$result->page = $this->loadTemplate('product/tags', array(

								'tags'       => $product->tags,
								'categories' => $product->categories

							));

						break;

						case 'gallery':

							$result->page = $this->loadTemplate('product/gallery', array(

								'images'  => $product->images,
								'product' => addslashes($product->name)

							));

						break;

						case 'versions':

							$result->page = $this->loadTemplate('product/versions', array(

								'ID'              => $product->ID,
								'name'            => $product->name,
								'versions'        => $product->versions,
								'onlySKU'         => $options->onlySKU,
								'options'         => $product->options,
								'requiredOptions' => $product->requiredOptions

							));

						break;

						case 'addtocart':

							$result->page = $this->loadTemplate('product/addtocart', array(

								'ID'              => $product->ID,
								'versions'        => $product->versions,
								'onlySKU'         => $options->onlySKU,
								'options'         => $product->options,
								'requiredOptions' => $product->requiredOptions

							));

						break;

						case 'description':

							$result->page = $product->description->long;

						break;

						case 'box':

							$result->page = $this->loadTemplate('catalog/box', array(

								'product' => $product

							));

						break;

						case 'rating':

							$result->page = $this->loadTemplate('product/rating', array(

								'average' => $product->ratingAverage,
								'count'   => $product->ratingCount

							));

						break;

					}

				}

			break;

		}

		return $result;

	}

	// crea la pagina dell'ordine da inviare
	private function getReceipt ($seller = FALSE) {

		$location = FALSE;

		if ($seller) {

			$ipinfo = json_decode(@file_get_contents('http://ipinfo.io/' . $_SERVER['REMOTE_ADDR'] . '/json'));

			if (isset($ipinfo->loc) && $ipinfo->hostname != 'localhost') {

				$location = $ipinfo->loc;
			}

		}

		$html = $this->loadTemplate('cart/receipt', array(

			'style'    => file_get_contents(RC_PLUGIN_FOLDER . '/resources/uikit/' . RCUtilities::$settings->general->UIkitVersion . '/css/uikit' . ((RCUtilities::$settings->general->style != 'flat') ? ('.' . RCUtilities::$settings->general->style) : '') . '.min.css'),
			'message'  => isset($this->cart->payment->method->type) ? RCUtilities::$config->{$this->cart->payment->method->type}->message : '',
			'seller'   => $seller,
			'location' => $location

		));

		$emogrifier = new \Pelago\Emogrifier();

		$emogrifier->setHtml($html);

		return $emogrifier->emogrifyBodyContent();

	}

	// prepara i link per il digital delivery
	private function sendDownload () {

		if (!isset($_POST['auth'])) {

			exit('Missing auth code.');

		}

		if ($_POST['auth'] != md5(RCUtilities::$config->amazon->secretKey . $_POST['email'] . $_POST['sku'])) {

			exit('Auth code is not correct.');

		}

		if  (!isset($_POST['email'])) {

			exit('Missing email address.');

		}

		if (!filter_var($_POST['email'], FILTER_VALIDATE_EMAIL)) {

			exit('Email address is not correct.');

		}

		if (RCUtilities::isBlacklisted($_POST['email']) !== TRUE) {

			exit('Email address has been blacklisted.');

		}

		if (!isset($_POST['sku'])) {

			exit('Missing product SKU.');

		}

		foreach ($this->catalog->products as $product) {

			foreach ($product->versions as $version) {

				if ($version->SKU == $_POST['sku']) {

					$item = $this->catalog->getProduct($product->ID)->setVersionAndOptions((object) array(

						'version' => $version->ID

					));

					if ($item === FALSE) {

						exit('Missing required options.');

					}

					$this->cart->reset();
					$this->cart->addItem($item);

					$this->cart->customer->emailAddress = $_POST['email'];

					$download = FALSE;

					if ($this->databaseEnabled) {

						$download = $this->database->addDownloads($this->cart);

					}

					else {

						$download = $this->cart->generatesDownloadLinks();

					}

					if ($download === FALSE) {

						exit('No downloadable files found.');

					}

					$order = new RCOrder();

					$order->setSubject(str_replace('%ORDER_ID%', $this->cart->ID, RCUtilities::$localization->order->title));
					$order->setAddress($_POST['email'], '');
					$order->setBodies($this->getReceipt(FALSE));

					if ($order->send() === TRUE) {

						exit('OK');

					}

					else {

						exit('Product not sent.');

					}

				}

			}

		}

		exit('Product not found.');

	}

	// invia la ricevuta di download avvenuto
	private function getDownloadsReceipt () {

		return $this->loadTemplate('download/receipt', array(

			'style'    => file_get_contents(RC_PLUGIN_FOLDER . '/resources/uikit/' . RCUtilities::$settings->general->UIkitVersion . '/css/uikit' . ((RCUtilities::$settings->general->style != 'flat') ? ('.' . RCUtilities::$settings->general->style) : '') . '.min.css'),
			'message'  => nl2br(sprintf(RCUtilities::$localization->download->receiptMessage, RCUtilities::$settings->digitalDelivery->expiration))

		));

	}

}

?>