<?php

class RCUtilities {
	
	static $config;
	static $settings;
	static $localization;
	
	static function init () {
		
		global $RCConfig;
		
		self::$config = json_decode(json_encode($RCConfig));
		
		self::$settings = self::getJSONData('settings.json');
		
		date_default_timezone_set(self::$settings->general->timeZone);
		
		self::$localization = self::getJSONData('localization.json');
		
		self::$settings->general->jQueryVersion = '2.2.4';
		self::$settings->general->UIkitVersion = '2.27.5';
		self::$settings->general->RCkitVersion = '4.0';
		self::$settings->cart = (object) array_merge((array) self::$settings->cart, (array) self::getJSONData('cart.json'));
		
		$coupons = json_decode(self::$config->cart->coupons);
		
		self::$settings->cart->acceptCouponCodes = FALSE;
		
		foreach (self::$settings->cart->costs as $cost) {
			
			if (isset($cost->couponRequired) && $cost->couponRequired) {
				
				self::$settings->cart->acceptCouponCodes = TRUE;
				
				foreach ($coupons as $id => $coupon) {
					
					if ($id == $cost->couponCode) {
						
						$cost->couponCode = $coupon;
						$cost->hasLimit = isset($cost->couponLimit);
						
						if (!isset($cost->couponLimit)) {
							
							$cost->couponLimit = self::$settings->database->enable ? 0 : 1;
							
						}
						
						break;
						
					}
					
				}
				
			}
			
		}
		
		self::$settings->product->defaultTaxes = array(0, 0, 0);
		
		if (isset(self::$settings->product->showDefaultTaxes) && self::$settings->product->showDefaultTaxes && !empty(self::$settings->cart->allowedCountries)) {
			
			foreach (self::$settings->cart->allowedCountries as $country) {
				
				if ($country->code == self::$settings->cart->defaultCountry) {
					
					self::$settings->product->defaultTaxes = array(isset($country->taxes) ? $country->taxes : 0, isset($country->taxes2) ? $country->taxes2 : 0, isset($country->taxes3) ? $country->taxes3 : 0);
					
					break;
					
				}
				
			}
			
		}
		
		self::$settings->payments = self::getJSONData('payment.json');
		
		if (session_id() == '') {
			
			$url = parse_url(self::$settings->general->siteURL);
		
			session_set_cookie_params(self::$settings->general->sessionValidity * 60 * 60 * 24, $url['path']);
			session_start();
			
		}
		
		register_shutdown_function('session_write_close');
		
		if (isset($_POST)) {
		
			$_POST = self::arrayStripslashes($_POST);
			
		}

		$dealers = (object) array(

			'loggedIn' => FALSE,
			'username' => '',
			'discount' => '',
			'applyTo'  => ''

		);
		
		self::$settings->dealers = self::loadSessionData('dealers', $dealers);

		if (isset($_POST['rc-username']) && isset($_POST['rc-password'])) {

			self::$settings->dealers->loggedIn = FALSE;
			
			if (!empty(self::$config->dealers->username1) && $_POST['rc-username'] == self::$config->dealers->username1 && md5($_POST['rc-password']) == self::$config->dealers->password1) {

				self::$settings->dealers->loggedIn = TRUE;
				self::$settings->dealers->username = self::$config->dealers->username1;
				self::$settings->dealers->discount = self::$config->dealers->discount1;
				self::$settings->dealers->applyTo = self::$config->dealers->applyTo1;

			}

			else if (!empty(self::$config->dealers->username2) && $_POST['rc-username'] == self::$config->dealers->username2 && md5($_POST['rc-password']) == self::$config->dealers->password2) {

				self::$settings->dealers->loggedIn = TRUE;
				self::$settings->dealers->username = self::$config->dealers->username2;
				self::$settings->dealers->discount = self::$config->dealers->discount2;
				self::$settings->dealers->applyTo = self::$config->dealers->applyTo2;

			}

			self::saveSessionData('dealers', self::$settings->dealers);

		}

		else {
			
			$pars = self::parseURL();
			
			if ($pars[0] == 'logout') {

				self::$settings->dealers = self::saveSessionData('dealers', $dealers);
			
			}

		}
		
		// fix for wrong HTTP protocol
		self::$settings->general->siteURL = self::fixHTTPProtocol(self::$settings->general->siteURL);
		self::$settings->general->pageURL = self::fixHTTPProtocol(self::$settings->general->pageURL);
		
	}
	
	static function arrayStripslashes ($array) {
		
		foreach ($array as &$value) {
			
			if (is_array($value)) {
				
				$value = self::arrayStripslashes($value);
				
			}
			
			else {
				
				$value = stripslashes($value);
				
			}
			
		}
			
		return $array;
		
	}
	
	static function getJSONData ($json) {
		
		return json_decode(file_get_contents((dirname(__FILE__) . '/../data/' . $json)));
		
	}
	
	static function getURL ($pars = array(), $replace = FALSE, $absolute = TRUE) {
		
		/*if (empty($pars)) {
			
			$url = ($absolute ? ('http://' . $_SERVER['HTTP_HOST']) : '') . $_SERVER['REQUEST_URI'];
			
		}
		
		else {
		
			if ($replace) {
			
				$url = $pars;
				
			}
			
			else {
			
				$url = self::parseURL();
				
				foreach ($pars as $key => $value) {
					
					$url[$key] = $value;
					
				}
				
			}
			
			if ($absolute) {
				
				$url = self::$settings->general->pageURL . '?' . implode('/', $url);
			
			}
				
			else {
				
				$splittedURL = parse_url(self::$settings->general->pageURL);
			
				$url = $splittedURL['path'] . '?' . implode('/', $url);
				
			}
		
		}*/
		
		if ($replace) {
		
			$url = $pars;
			
		}
		
		else {
		
			$url = self::parseURL();
			
			foreach ($pars as $key => $value) {
				
				$url[$key] = $value;
				
			}
			
		}
		
		if ($absolute) {
			
			$url = self::$settings->general->pageURL . '?' . implode('/', $url);
		
		}
			
		else {
			
			$splittedURL = parse_url(self::$settings->general->pageURL);
		
			$url = $splittedURL['path'] . '?' . implode('/', $url);
			
		}
		
		if ($absolute) {
			
			return self::fixHTTPProtocol($url);
			
		}
		
		else {
		
			return $url;
		
		}
		
	}
	
	static function fixHTTPProtocol ($url) {
		
		if ((isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') || self::$settings->general->forceHTTPS) {
			
			return preg_replace("/^http:/i", "https:", $url);
			
		}
		
		else {
			
			return preg_replace("/^https:/i", "http:", $url);
			
		}
		
	}
	
	static function parseURL () {
		
		return @explode('/', trim(urldecode($_SERVER['QUERY_STRING']), '/'));
		
	}
	
	static function loadSessionData ($name, $default = FALSE) {
		
		if (isset($_SESSION['rapidcartpro'][$name])) {
			
			return $_SESSION['rapidcartpro'][$name];
			
		}
		
		else {
			
			return self::saveSessionData($name, $default);
			
		}
		
	}
	
	static function redirectToURL ($url) {
		
		if (ob_get_contents()) {
			
			ob_end_clean();
			
		}
			
		header('Location: ' . $url);
		
		exit();
		
	}
	
	static function saveSessionData ($name, $data) {
		
		$_SESSION['rapidcartpro'][$name] = $data;
		
		return $data;
		
	}
	
	static function toCurrency ($amount, $sign = FALSE, $freeLabel = TRUE) {
		
		if ($amount == 0 && $freeLabel) {
			
			return self::$localization->general->free;
			
		}
		
		$string = (number_format($amount, self::$localization->currency->decimals) >= 0) ? ($sign ? '+ ' : '') : '- ';
		
		if (self::$localization->currency->symbolPosition == 'left') {
			
			$string .= self::$localization->currency->symbol . self::$localization->currency->symbolSeparator;
			
		}
		
		$string .= number_format(abs($amount), self::$localization->currency->decimals, self::$localization->currency->decimalSeparator, self::$localization->currency->thousandsSeparator);
		
		if (self::$localization->currency->symbolPosition == 'right') {
			
			$string .= self::$localization->currency->symbolSeparator . self::$localization->currency->symbol;
			
		}
		
		return $string;
		
	}
	
	static function toPercentage ($amount, $sign = FALSE, $decimals = TRUE) {
		
		$string = ($amount >= 0) ? ($sign ? '+ ' : '') : '- ';
		$string .= number_format(abs($amount), self::$localization->currency->decimals * $decimals, self::$localization->currency->decimalSeparator, self::$localization->currency->thousandsSeparator);
		$string .= '%';
		
		return $string;
		
	}
	
	static function getLocalizedMonthsArray () {
		
		$months = '';
	
		foreach (self::$localization->dateTime->months as $month) {
			
			$months .= '\'' . addslashes($month) . '\', ';
			
		}

		return '[' . substr($months, 0, -2) . ']';
		
	}
	
	static function getLocalizedWeekdaysArray () {
		
		$weekdays = '';
	
		foreach (self::$localization->dateTime->weekdays as $weekday) {
			
			$weekdays .= '\'' . addslashes($weekday) . '\', ';
			
		}

		return '[' . substr($weekdays, 0, -2) . ']';
		
	}
	
	static function getLocalizedDate ($date) {
		
		$search = array('DD', 'MM', 'YYYY');
		$replace = array('d', 'm', 'Y');
		
		return date(str_replace($search, $replace, self::$localization->dateTime->dateFormat), strtotime($date));
				
	}
	
	static function convertLocalizedDate ($string) {
		
		switch (self::$localization->dateTime->dateFormat) {
			
			case 'YYYY-MM-DD':
			
				list($year, $month, $day) = explode('-', $string);
				
			break;
			
			case 'YYYY/MM/DD':
			
				list($year, $month, $day) = explode('/', $$string);
				
			break;
			
			case 'YYYY.MM.DD':
			
				list($year, $month, $day) = explode('.', $string);
				
			break;
			
			case 'DD-MM-YYYY':
			
				list($day, $month, $year) = explode('-', $string);
				
			break;
			
			case 'DD/MM/YYYY':
			
				list($day, $month, $year) = explode('/', $string);
				
			break;
				
			case 'DD.MM.YYYY':
			
				list($day, $month, $year) = explode('.', $string);
				
			break;
				
			case 'MM-DD-YYYY':
			
				list($month, $day, $year) = explode('-', $string);
				
			break;
			
			case 'MM/DD/YYYY':
			
				list($month, $day, $year) = explode('/', $string);
				
			break;
			
			case 'MM.DD.YYYY':
			
				list($month, $day, $year) = explode('.', $string);
				
			break;
			
			default:
			
				list($year, $month, $day) = explode('-', $string);
				
			break;
			
		}
		
		return mktime(0, 0, 0, $month, $day, $year);
		
	}
	
	static function formatBytes ($bytes) {
		
		if ($bytes >= 1073741824) {
			
			$bytes = number_format($bytes / 1073741824, 2) . ' GB';
			
		}
		
		else if ($bytes >= 1048576) {
			
			$bytes = number_format($bytes / 1048576, 2) . ' MB';
			
		}
		
		else if ($bytes >= 1024) {
			
			$bytes = number_format($bytes / 1024, 2) . ' KB';
		
		}
		
		else if ($bytes > 1) {
			
			$bytes = $bytes . ' bytes';
		
		}
		
		else if ($bytes == 1) {
			
			$bytes = $bytes . ' byte';
		
		}
		
		else {

			$bytes = '0 bytes';
	
		}
		
		return $bytes;
		
	}
	
	static function isBlacklisted ($email = '') {
		
		$list = json_decode(self::$config->general->blacklist);
		
		if (!empty($list)) {
			
			foreach ($list as $row) {
					
				if ($row->type == 'IP' && $row->value == $_SERVER['REMOTE_ADDR']) {
				
					return 'IP_BLACKLISTED';
					
				}
				
				else if (!empty($email) && $row->type == 'email' && $row->value == $email) {
					
					return 'EMAIL_BLACKLISTED';
					
				}
				
			}
			
		}
		
		return TRUE;
		
	}
	
	static function dataEncrypt ($string) {

		$key = self::$config->amazon->secretKey;
		
		for ($i = 0; $i < strlen($string); $i++) {
		
			$string[$i] = chr(ord($string[$i]) ^ ord($key[$i % strlen($key)]));
			
		}
		
		return $string;
		
	}
	
	static function sendWebhookData ($url, $data, $auth = FALSE) {
		
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
		curl_setopt($ch, CURLOPT_FAILONERROR, TRUE);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15); 
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
		
			'Content-Type: application/json'
			
		));
		
		try {
		
			$data = json_decode(curl_exec($ch));
			$error = curl_error($ch);
			
		}
		
		catch (Exception $e) {
			
			$data = '';
			$error = $e->getMessage();
			
		}
		
		return (object) array(
		
			'data'  => $data,
			'error' => $error
			
		);
		
	}
	
	static function optionsToList ($options) {
		
		$list = array();
		
		foreach ($options as $option) {
			
			$values = array();
			
			if (in_array($option->type, array(
			
				'text',
				'textarea',
				'date',
				'time',
				'number'
			
			))) {
				
				if (!empty($option->values[0])) {
				
					$values = $option->values;
					
				}
				
			}
			
			else {
				
				foreach ($option->values as $value) {
					
					if ($value->selected) {
						
						$values[] = $value->value;
						
					}
					
				}
				
			}
			
			if (!empty($values)) {
				
				$list[] = (object) array(
				
					'name'   => $option->name,
					'values' => $values
					
				);
				
			}
			
		}
		
		return $list;
		
	}
	
	static function sendPushNotification ($orderTotal, $orderName, $orderID) {
		
		$tags = (object) array(
			
			'key'      => 'RCPNID',
			'relation' => '=',
			'value'    => md5('RCPNID' . RCUtilities::$settings->general->pageURL)
			
		);
		
		$data = array(
		
			'app_id'   => '76de3f9c-55db-4891-b110-792deb83a076',
			'url'      => RCUtilities::getURL(array(
			
				0 => 'admin',
				1 => 'order',
				2 => $orderID
				
			), TRUE),
			'tags'     => array($tags),
			'contents' => array(
			
				'en' => sprintf(RCUtilities::$localization->admin->notificationsOrderMessage, self::toCurrency($orderTotal, FALSE, FALSE), $orderName)
				
			)
			
		);
		
		try {
			
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, 'https://onesignal.com/api/v1/notifications');
			curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json', 'Authorization: Basic Nzk1NmM3OWEtZWQ4ZS00MTVlLWE1NjUtN2I1NGNhZTJmNGVk'));
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
			curl_setopt($ch, CURLOPT_HEADER, FALSE);
			curl_setopt($ch, CURLOPT_POST, TRUE);
			curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
			
			$response = curl_exec($ch);
			curl_close($ch);
			
			return $response;
			
		}
		
		catch (Exception $e) {}
		
	}
	
}

?>