#!/bin/bash
set -o errexit

echo ------------------------------------------
echo  env variables
echo ------------------------------------------
echo TARGET_BUILD_DIR = $TARGET_BUILD_DIR
echo DEPLOYMENT_LOCATION = $DEPLOYMENT_LOCATION
echo INSTALL_PATH = $INSTALL_PATH
echo SKIP_INSTALL = $SKIP_INSTALL
echo OBJROOT = $OBJROOT
echo CONFIGURATION_BUILD_DIR = $CONFIGURATION_BUILD_DIR
echo TARGET_TEMP_DIR = $TARGET_TEMP_DIR
echo PRODUCT_NAME = $PRODUCT_NAME
echo PROJECT_NAME = $PROJECT_NAME
echo TARGET_NAME = $TARGET_NAME
echo ------------------------------------------

# editable vars #
AC_USERNAME="gilberto.defaveri@mac.com"
DMG_NAME="RapidCartPro"
DMG_FOLDER="/Users/gibo/Code/Cocoa/$DMG_NAME/Installer"
#################

# Workaround for product placed in UninstalledProducts folder
ARCHIVED_BUILD_FOLDER=$OBJROOT/UninstalledProducts/macosx
unlink $TARGET_BUILD_DIR/"$PRODUCT_NAME.rapidweaverplugin"
mv $ARCHIVED_BUILD_FOLDER/"$PRODUCT_NAME.rapidweaverplugin" $TARGET_BUILD_DIR/"$PRODUCT_NAME.rapidweaverplugin"

# Prepare DMG
RCPSTACKSTMP=`mktemp -d -t 'RCPSTACKSTMP'`

# Download latest stacks
curl -o "$RCPSTACKSTMP/RapidCartProStacks.zip" https://tls.4gnd.com/rw/rapidcartpro/stacksupdate/RapidCartProStacks.zip
unzip "$RCPSTACKSTMP/RapidCartProStacks.zip" -d "$RCPSTACKSTMP"

# Mount image
hdiutil attach -mountpoint "/Volumes/$DMG_NAME" "$DMG_FOLDER/dmg rw/$DMG_NAME.dmg"

# Copy plugin in place
rm -rf "/Volumes/$DMG_NAME/$PRODUCT_NAME.rapidweaverplugin"
cp -r "$BUILT_PRODUCTS_DIR/$PRODUCT_NAME.rapidweaverplugin" "/Volumes/$DMG_NAME"

# Move stacks in place and clean up
rm -rf "/Volumes/$DMG_NAME/Extras/Stacks"
mkdir "/Volumes/$DMG_NAME/Extras/Stacks"
for stack in `ls "$RCPSTACKSTMP" | grep .stack`; do
    echo "Moving $stack..."
    mv "$RCPSTACKSTMP/$stack" "/Volumes/$DMG_NAME/Extras/Stacks"
done
rm -rf "$RCPSTACKSTMP"

# Create read-onlt DMG
hdiutil detach "/Volumes/$DMG_NAME"
rm -rf "$DMG_FOLDER/$DMG_NAME.dmg"
hdiutil convert "$DMG_FOLDER/dmg rw/$DMG_NAME.dmg" -format UDBZ -o "$DMG_FOLDER/$DMG_NAME.dmg"

# Here's how to add an item to login Keychain
#$ security add-generic-password -a "gilberto.defaveri@mac.com" -w <password> -s "AC_PASSWORD"

echo "########################"
echo Starting notarization...
echo "########################"
echo xcrun altool --notarize-app --primary-bundle-id "$PRODUCT_NAME.dmg" --username "$AC_USERNAME" --password "@keychain:AC_PASSWORD" --file "$DMG_FOLDER/$DMG_NAME.dmg"
xcrun altool --notarize-app --primary-bundle-id "$PRODUCT_NAME.dmg" --username "$AC_USERNAME" --password "@keychain:AC_PASSWORD" --file "$DMG_FOLDER/$DMG_NAME.dmg"
echo "###########################"
echo "Submitted for notarization."
echo "###########################"

PROCEEDTOSTAPLING="0"
while [ $PROCEEDTOSTAPLING -ne "1" ]; do
    echo    # (optional) move to a new line
    echo "(C)heck for notarizaion status"
    echo "(P)roceed to stapling"
    echo "(V)iew notarization log"
    echo "(A)bort"
    read -p "Choose (c), (p), (v) or (a)? " -n 1 -r
    echo    # (optional) move to a new line
    if [[ $REPLY =~ ^[Aa]$ ]]
    # (A)bort
    then
        exit
    fi
    if [[ $REPLY =~ ^[Cc]$ ]]
    # (C)heck
    then
        echo xcrun altool --notarization-history 0 -u "$AC_USERNAME" -p "@keychain:AC_PASSWORD"
        xcrun altool --notarization-history 0 -u "$AC_USERNAME" -p "@keychain:AC_PASSWORD"
    fi
    if [[ $REPLY =~ ^[Vv]$ ]]
    # (V)iew
    then
        read -p "Please type UUID for the request? " -r
        echo    # (optional) move to a new line
        echo xcrun altool --notarization-info $REPLY -u "$AC_USERNAME" -p "@keychain:AC_PASSWORD"
        xcrun altool --notarization-info $REPLY -u "$AC_USERNAME" -p "@keychain:AC_PASSWORD"
    fi
    if [[ $REPLY =~ ^[Pp]$ ]]
    # (P)roceed
    then
        PROCEEDTOSTAPLING="1"

        echo "Stapling..."

        ###########################
        # Create appcast and update
        ###########################

        APPCAST_FILENAME=appcast.txt

        VERSION=$(defaults read "$BUILT_PRODUCTS_DIR/$PRODUCT_NAME.rapidweaverplugin/Contents/Info" CFBundleVersion)
        SHORTVERSIONSTRING=$(defaults read "$BUILT_PRODUCTS_DIR/$PRODUCT_NAME.rapidweaverplugin/Contents/Info" CFBundleShortVersionString)
        DOWNLOAD_BASE_URL="https://tls.4gnd.com/rw/rapidcartpro/rwupdate"
        RELEASENOTES_URL="$DOWNLOAD_BASE_URL/release-notes.php#version-$SHORTVERSIONSTRING"

        ARCHIVE_FILENAME="$PROJECT_NAME-$SHORTVERSIONSTRING-($VERSION).zip"
        DOWNLOAD_URL="$DOWNLOAD_BASE_URL/$ARCHIVE_FILENAME"
        KEYCHAIN_PRIVKEY_NAME="Sparkle RapidCartPro Private Key"

        echo Creating zip archive for update...
        WD=$PWD
        cd "$BUILT_PRODUCTS_DIR"
        rm -f "$PROJECT_NAME"*.zip
        ######################
        echo xcrun stapler staple "$PRODUCT_NAME.rapidweaverplugin"
        xcrun stapler staple "$PRODUCT_NAME.rapidweaverplugin"
        echo "#############################"
        echo "Validating stapled package..."
        echo "#############################"
        echo xcrun stapler validate "$PRODUCT_NAME.rapidweaverplugin"
        xcrun stapler validate "$PRODUCT_NAME.rapidweaverplugin"
        ######################
        zip -qr --symlinks "$ARCHIVE_FILENAME" "$PRODUCT_NAME.rapidweaverplugin"

        TEMP_PRIV_KEY=$WD/tmp_priv_key
        security find-generic-password -g -s "$KEYCHAIN_PRIVKEY_NAME" 2>&1 1>/dev/null | perl -pe '($_) = /"(.+)"/; s/\\012/\n/g; s/\\011//g' > $TEMP_PRIV_KEY.plist

        echo defaults read $TEMP_PRIV_KEY NOTE > $TEMP_PRIV_KEY.plain
        defaults read $TEMP_PRIV_KEY NOTE > $TEMP_PRIV_KEY.plain

        SIZE=$(stat -f %z "$ARCHIVE_FILENAME")
        PUBDATE=$(date +"%a, %d %b %G %T %z")
        SIGNATURE=$(
        openssl dgst -sha1 -binary < "$ARCHIVE_FILENAME" \
        | openssl dgst -dss1 -sign <(cat "$TEMP_PRIV_KEY.plain") \
        | openssl enc -base64
        )

        rm $TEMP_PRIV_KEY.plist
        rm $TEMP_PRIV_KEY.plain

        [ $SIGNATURE ] || { echo Unable to load signing private key with name "'$KEYCHAIN_PRIVKEY_NAME'" from keychain; false; }

        cat > "$BUILT_PRODUCTS_DIR/$APPCAST_FILENAME" <<EOF
        <item>
            <title>$PRODUCT_NAME $SHORTVERSIONSTRING</title>
            <sparkle:releaseNotesLink>$RELEASENOTES_URL</sparkle:releaseNotesLink>
            <pubDate>$PUBDATE</pubDate>
            <enclosure
                url="$DOWNLOAD_URL"
                sparkle:version="$VERSION"
                sparkle:shortVersionString="$SHORTVERSIONSTRING"
                type="application/octet-stream"
                length="$SIZE"
                sparkle:dsaSignature="$SIGNATURE"
            />
        </item>
EOF

        ############################
        # Re-create DMG
        ############################
        echo Recreating DMG with stapled content...

        # Mount image
        hdiutil attach -mountpoint "/Volumes/$DMG_NAME" "$DMG_FOLDER/dmg rw/$DMG_NAME.dmg"

        # Copy plugin in place
        rm -rf "/Volumes/$DMG_NAME/$PRODUCT_NAME.rapidweaverplugin"
        cp -r "$BUILT_PRODUCTS_DIR/$PRODUCT_NAME.rapidweaverplugin" "/Volumes/$DMG_NAME"

        # Create read-onlt DMG
        hdiutil detach "/Volumes/$DMG_NAME"
        rm -rf "$DMG_FOLDER/$DMG_NAME.dmg"
        hdiutil convert "$DMG_FOLDER/dmg rw/$DMG_NAME.dmg" -format UDBZ -o "$DMG_FOLDER/$DMG_NAME.dmg"

    fi
done

#########################
open "$BUILT_PRODUCTS_DIR"
open "$DMG_FOLDER"
#########################
